<?php

namespace Ecommerce\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Teknisa\Libs\Util\Utilities as LibUtilities;

class AdministradorController extends \Teknisa\Libs\Controller\CRUD {
    
    public function __construct($crudService, $environment, $connection, $serviceEmail, $criptografia, $entityManager, $newCode, LibUtilities $utilities) {
        $this->environment   = $environment;
        $this->connection    = $connection;
        $this->serviceEmail  = $serviceEmail;
        $this->criptografia  = $criptografia;
        $this->entityManager = $entityManager;
        $this->newCode       = $newCode;
        $this->utilities     = $utilities;
    }
    
    public function validacoesAcesso(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            if(isset($row["__is_new"]) && $row["__is_new"] == true) {
                $query = $this->connection->prepare(
                    "SELECT DSLOGINECO FROM DADOSFORNECOMMERCE WHERE CDFORNECED = :P_CDFORNECED AND DSLOGINECO = :P_DSLOGINECO"
                );
                $query->execute( array(
                    "P_CDFORNECED" => $row["CDFORNECED"],
                    "P_DSLOGINECO" => $row["DSLOGINECO"]
                ));
                $dataSet = $query->fetchAll();
                if(sizeof($dataSet) > 0) {
                    throw new \Exception("Login já cadastrado");
                }
            }
            
            $query = $this->connection->prepare(
                "SELECT DSLOGINECO, IDTPIJURFORN, NRINSJURFORN, NMFANTFORN, NMRAZSOCFORN
                    FROM DADOSFORNECOMMERCE, FORNECEDOR
                    WHERE DADOSFORNECOMMERCE.CDFORNECED = FORNECEDOR.CDFORNECED
                        AND DADOSFORNECOMMERCE.CDFORNECED <> :P_CDFORNECED
                        AND DSLOGINECO = :P_DSLOGINECO"
            );
            $query->execute(array(
                "P_CDFORNECED" => $row["CDFORNECED"],
                "P_DSLOGINECO" => $row["DSLOGINECO"]
            ));
            $dataSet = $query->fetchAll();
            $response->addDataSet(new DataSet('fornecedoresRelacionados', $dataSet));
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), 1));
        }
        
    }
    
    public function verficaOperadorExiste($email) {
        $query = $this->connection->prepare(
            "SELECT CDOPERFORN FROM DADOSFORNECOMMERCE WHERE DSLOGINECO = :DSLOGINECO"
        );
        $query->execute(array("DSLOGINECO"  => $email));
        $cdoperador = $query->fetchAll();
        
        if(count($cdoperador) > 0) {
            $cdoperador = $cdoperador[0]['CDOPERFORN'];
        } else {
            $cdoperador = false;
        }
        
        return $cdoperador;
    }
    
    protected function verificaPerfilAcessoFornecedor() {
        $query = $this->connection->prepare(
            "SELECT * FROM PERFILACESSO WHERE UPPER(NMPERFILACESSO) LIKE '%FORNECEDOR%'"
        );
        $query->execute();
        return $query->fetchAll();
    }
    
    protected function insereFornecedorPerfilAcesso() {
        // Seleciona código máximo da tabela PERFILACESSO
        $query = $this->connection->prepare(
            "SELECT MAX(NRPERFILACESSO) as MAXNRPERFILACESSO FROM PERFILACESSO"
        );
        $query->execute();
        $codigo = $query->fetchAll()[0]["MAXNRPERFILACESSO"];
        
        // Insere fornecedor na tabela PERFILACESSO
        $query = $query = $this->connection->prepare(
            "INSERT INTO PERFILACESSO
                (NRPERFILACESSO, NMPERFILACESSO, NRORG, IDATIVO, CDOPERINCLUSAO, CDOPERULTATU)
            VALUES
                (:NRPERFILACESSO, 'FORNECEDOR', :NRORG, 'S', :CDOPERINCLUSAO, :CDOPERULTATU)"
        );
        $query->execute(array(
            "NRPERFILACESSO" => $codigo + 10,
            "NRORG"          => $this->environment->getNrOrg(),
            "CDOPERINCLUSAO" => $this->environment->getCdOperador(),
            "CDOPERULTATU"   => $this->environment->getCdOperador()
        ));
        return $codigo + 10;
    }
    
    protected function novoCodigoPerfacessparc() {
        return (int)$this->newCode->getCodeByTableName('PERFACESSPARC', $this->environment->getNrOrg());
    }
    
    protected function inserePerfacessparc($codigoParcNegocio, $novoCodigoPerfil, $nrperfilacesso) {
        $query = $query = $this->connection->prepare(
            "INSERT INTO PERFACESSPARC
              (NRPERFACESSPARC, NRPARCNEGOCIO, NRPERFILACESSO, DTINCLUSAO)
            VALUES
              (:NRPERFACESSPARC, :NRPARCNEGOCIO, :NRPERFILACESSO, SYSDATE)"
        );
        $query->execute(array(
            "NRPERFACESSPARC" => $novoCodigoPerfil,
            "NRPARCNEGOCIO"   => $codigoParcNegocio,
            "NRPERFILACESSO"  => $nrperfilacesso
        ));
    }
    
    public function retornaParcnegocio($cdoperador) {
        $query = $this->connection->prepare(
            "SELECT NRPARCNEGOCIO FROM PARCNEGOCIO WHERE  NRINSCRICAOPARC LIKE '%" . $cdoperador . "'"
        );
        $query->execute();
        $nrparcnegocio = $query->fetchAll();
        
        if(count($nrparcnegocio) > 0) {
            $nrparcnegocio = $nrparcnegocio[0]['NRPARCNEGOCIO'];
        } else {
            $nrparcnegocio = false;
        }
        
        return $nrparcnegocio;
    }
    
    public function salvarParceiro($row) {
        $nrperfilacesso = "";

        $codigoParcNegocio = self::retornaParcnegocioOperador($row["DSLOGINECO"]);
        $perfilAcesso = $this->verificaPerfilAcessoFornecedor();

        if(sizeof($perfilAcesso) == 0) {
            $nrperfilacesso = $this->insereFornecedorPerfilAcesso();
        } else {
            $nrperfilacesso = $perfilAcesso[0]["NRPERFILACESSO"];
        }
        
        $novoCodigoPerfil = $this->novoCodigoPerfacessparc();
        $this->inserePerfacessparc($codigoParcNegocio, $novoCodigoPerfil, $nrperfilacesso);
    }
    
    public function retornaParcnegocioOperador($dsemailoper) {
        
        $query = $this->connection->prepare(
            "SELECT NRPARCNEGOCIO FROM OPERADOR WHERE DSEMAILOPER = :DSEMAILOPER AND NRORG = :NRORG ORDER BY DTINCLUSAO DESC"
        );
        $query->execute(array("DSEMAILOPER"  => $dsemailoper, "NRORG" => $this->environment->getNrOrgTrab()));
        $nrparcnegocio = $query->fetchAll();
        
        if(count($nrparcnegocio) > 0) {
            $nrparcnegocio = $nrparcnegocio[0]['NRPARCNEGOCIO'];
        } else {
            $nrparcnegocio = false;
        }
        
        return $nrparcnegocio;
        
    }
    
    public function retornaCdfilial() {
        
        $query = $this->connection->prepare(
            "SELECT MAX(CDFILIAL) CDFILIAL FROM PARASUPRI"
        );
        
        $query->execute();
        $cdfilia = $query->fetchAll();
        
        if(count($cdfilia) > 0) {
            $cdfilia = $cdfilia[0]['CDFILIAL'];
        } else {
            $cdfilia = false;
        }
        
        return $cdfilia;
        
    }
    
    public function salvarNovoLogin(DTO\Request $request, DTO\Response $response) {
        try {
            $this->entityManager->beginTransaction();
            
            $row = $request->getRow();
            $this->geraNumOperador($row);
            
            $query = $this->connection->prepare(
                "INSERT INTO DADOSFORNECOMMERCE
                   (CDFORNECED, DSLOGINECO, CDSENHAWEB, CDSENHA, IDACESSOCOT, IDACESSOAF,
                    IDACESSOCR, IDADMINISTRADOR, DSEMAILECO, NRORG, NMUSUARIOECO, CDOPERFORN)
                VALUES 
                   (:CDFORNECED, :DSLOGINECO, :CDSENHAWEB, :CDSENHA, :IDACESSOCOT, :IDACESSOAF,
                     :IDACESSOCR, :IDADMINISTRADOR, :DSEMAILECO, :NRORG, :NMUSUARIOECO, :CDOPERFORN)"
            );
            $query->execute(array(
                "CDFORNECED"      => $row["CDFORNECED"],
                "DSLOGINECO"      => $row["DSLOGINECO"],
                "CDSENHAWEB"      => $this->criptografia->retornaSenhaWeb($row["CDSENHAWEB"]),
                "CDSENHA"         => $this->utilities->getDelphiPasswordEncrypted($row["CDSENHAWEB"]),
                "IDACESSOCOT"     => $row["IDACESSOCOT"],
                "IDACESSOAF"      => $row["IDACESSOAF"],
                "IDACESSOCR"      => $row["IDACESSOCR"],
                "IDADMINISTRADOR" => 'N',
                "DSEMAILECO"      => $row["DSEMAILECO"],
                "NMUSUARIOECO"    => $row["NMUSUARIOECO"],
                "CDOPERFORN"      => $row["CDOPERFORN"],
                "NRORG"           => $this->environment->getNrOrgTrab()
            ));
            
            
            $this->entityManager->flush();
            $this->entityManager->commit();
            
        } catch (\Exception $e){
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 1));
        }
    } 
    
    public function checkTriggerExist() 
    {
        $query = $this->connection->prepare(
            "SELECT 
                COUNT(TRIGGER_NAME) as TRIG
            FROM 
                user_triggers
            WHERE 
                trigger_name = 'GERA_PARC_OPERADOR_SIMPLES'"
        );
        
        $query->execute();
        
        $dados = $query->fetchAll();
        return $dados[0]['TRIG'];
    }
    
    public function setTrigger($value, $check) 
    {
        try {
            if ($check) {
                if ($value) {
                    $query = $this->connection->prepare(
                        "ALTER TRIGGER GERA_PARC_OPERADOR_SIMPLES ENABLE"
                    );
                } else {
                    $query = $this->connection->prepare(
                        "ALTER TRIGGER GERA_PARC_OPERADOR_SIMPLES DISABLE"
                    );
                }
                
                $query->execute();
            }
        } catch (Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function createTriggerBody($row) 
    {
        $nrParcNegocio = $this->getMaxParcNegocio();
        $nrSequencial = $this->getMaxNrSequencial();
        
        $qtde = $nrParcNegocio > $nrSequencial ? ($nrParcNegocio - $nrSequencial) + 1 : 1;
        
        $row["NRPARCNEGOCIO"] = (int)$this->newCode->getCodeByTableName('PARCNEGOCIO', $this->environment->getNrOrg(), $qtde);
        $row["CDOPERADOR"] = '1099' . str_pad(mt_rand(1,99999999),8,'0',STR_PAD_LEFT);
        $row["NMOPERADOR"] = $this->getNmOperador($row);
        
        $this->insereParcNegocio($row);
    }
    
    public function getMaxParcNegocio() 
    {
        $query = $this->connection->prepare(
            "SELECT NVL(MAX(NRPARCNEGOCIO), 0) AS NRPARCNEGOCIO FROM PARCNEGOCIO"
        );
        
        $query->execute();
        
        $dados = $query->fetchAll();
        return $dados[0]['NRPARCNEGOCIO'];
    }
    
    public function getMaxNrSequencial() 
    {
        $query = $this->connection->prepare(
            "SELECT NVL(MAX(NRSEQUENCIAL), 0) as NRSEQUENCIAL FROM NOVOCODIGO WHERE CDCONTADOR = 'PARCNEGOCIO'"
        );
        
        $query->execute();
        
        $dados = $query->fetchAll();
        return floatval($dados[0]['NRSEQUENCIAL']);
    }
    
    public function getNmOperador($row) 
    {
        $query = $this->connection->prepare(
            "SELECT 
                MAX(NMFANTFORN) AS NMFANTFORN
            FROM 
                DADOSFORNECOMMERCE D, 
                FORNECEDOR F
            WHERE 
                D.CDFORNECED = :CDFORNECED AND 
                D.CDFORNECED = F.CDFORNECED"
        );
        
        $query->execute(array(
            "CDFORNECED"  => $row["CDFORNECED"]
        ));
        
        $dados = $query->fetchAll();
        return $dados[0]['NMFANTFORN'];
    }
    
    public function insereParcNegocio($row) 
    {
        $query = $this->connection->prepare(
            "INSERT INTO 
                PARCNEGOCIO ( 
                    NRPARCNEGOCIO, CDTIPOPARCPRINCIPAL, IDATIVO, DTINCLUSAO, 
                    CDTIPOINSCRICAO, IDPESSOAFISICA, IDINSTITUICAO, 
                    NRINSCRICAOPARC, NMPRINCIPALPARC, NMSECUNDARIPARC, NRORG
                ) 
                VALUES (
                    :NRPARCNEGOCIO, 'OPERADOR', 'S', SYSDATE, 'CPF', 'S', 'N',
                    :CDOPERADOR, :NMOPERADOR, :NMOPERADOR, :NRORG
                )"
        );
        
        $query->execute(array(
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "CDOPERADOR"    => $row["CDOPERADOR"],
            "NMOPERADOR"    => $row["NMOPERADOR"],
            "NRORG"         => $this->environment->getNrOrg()
        ));
    }
    
    public function salvarEdicaoLogin(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            if($row["SENHA_ALTERADA"] == "S") {
                $query = $this->connection->prepare(
                    "UPDATE DADOSFORNECOMMERCE
                    SET CDSENHAWEB      = :CDSENHAWEB,
                        CDSENHA         = :CDSENHA,
                        IDACESSOCOT     = :IDACESSOCOT,
                        IDACESSOAF      = :IDACESSOAF,
                        IDACESSOCR      = :IDACESSOCR,
                        IDADMINISTRADOR = :IDADMINISTRADOR,
                        DSEMAILECO      = :DSEMAILECO,
                        NMUSUARIOECO    = :NMUSUARIOECO
                    WHERE DSLOGINECO    = :DSLOGINECO
                        AND CDFORNECED  = :CDFORNECED"
                );
                $query->execute(array(
                    "CDSENHAWEB"      => $this->criptografia->retornaSenhaWeb($row["CDSENHAWEB"]),
                    "CDSENHA"         => $this->utilities->getDelphiPasswordEncrypted($row["CDSENHAWEB"]),
                    "IDACESSOCOT"     => $row["IDACESSOCOT"],
                    "IDACESSOAF"      => $row["IDACESSOAF"],
                    "IDACESSOCR"      => $row["IDACESSOCR"],
                    "IDADMINISTRADOR" => 'N',
                    "DSEMAILECO"      => $row["DSEMAILECO"],
                    "DSLOGINECO"      => $row["DSLOGINECO"],
                    "NMUSUARIOECO"    => $row["NMUSUARIOECO"],
                    "CDFORNECED"      => $row["CDFORNECED"]
                ));
                $response->addDataSet(new DataSet('senhaMD5', [md5($row["CDSENHAWEB"])]));
            } else {
                $query = $this->connection->prepare(
                    "UPDATE DADOSFORNECOMMERCE
                    SET IDACESSOCOT     = :IDACESSOCOT,
                        IDACESSOAF      = :IDACESSOAF,
                        IDACESSOCR      = :IDACESSOCR,
                        IDADMINISTRADOR = :IDADMINISTRADOR,
                        DSEMAILECO      = :DSEMAILECO,
                        NMUSUARIOECO    = :NMUSUARIOECO
                    WHERE DSLOGINECO    = :DSLOGINECO
                        AND CDFORNECED  = :CDFORNECED"
                );
                $query->execute(array(
                    "IDACESSOCOT"     => $row["IDACESSOCOT"],
                    "IDACESSOAF"      => $row["IDACESSOAF"],
                    "IDACESSOCR"      => $row["IDACESSOCR"],
                    "IDADMINISTRADOR" => 'N',
                    "DSEMAILECO"      => $row["DSEMAILECO"],
                    "DSLOGINECO"      => $row["DSLOGINECO"],
                    "NMUSUARIOECO"    => $row["NMUSUARIOECO"],
                    "CDFORNECED"      => $row["CDFORNECED"]
                ));
                $response->addDataSet(new DataSet('senhaMD5', [md5($row["CDSENHAWEB"])]));
            }
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvarLoginRelacionado(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $query = $this->connection->prepare(
                "UPDATE DADOSFORNECOMMERCE SET CDSENHAWEB = :CDSENHAWEB, CDSENHA = :CDSENHA WHERE DSLOGINECO = :DSLOGINECO"
            );
            $query->execute(array(
                "CDSENHAWEB"   => $this->criptografia->retornaSenhaWeb($row["CDSENHAWEB"]),
                "CDSENHA"      => $this->utilities->getDelphiPasswordEncrypted($row["CDSENHAWEB"]),
                "DSLOGINECO"   => $row["DSLOGINECO"]
            ));
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function deleteControlsenha ($row) {

        $query = $this->entityManager->getConnection()->prepare(
            "DELETE FROM CONTROLSENHA WHERE CDOPERADOR IN( SELECT CDOPERADOR FROM OPERADOR WHERE CDOPERADOR LIKE '1099%' AND DSEMAILOPER = :DSLOGINECO)"
        );
        
        $query->execute(array(
            "DSLOGINECO" => $row["DSLOGINECO"]
        ));
    
    }
    
    public function deletePerfilAcessoParc($cdoperador) {
        $nrparcnegocio = self::retornaParcnegocio($cdoperador);
        
        if($nrparcnegocio) {
            $query = $this->entityManager->getConnection()->prepare(
                "DELETE FROM PERFACESSPARC WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO"
            );
            $query->execute(array(
                "NRPARCNEGOCIO" => $nrparcnegocio
            ));
        }
        
    }
    
    public function excluirAcesso(DTO\Request $request, DTO\Response $response) {
        try {
            $this->entityManager->beginTransaction();
            
            $row = $request->getRow();
            
            $cdoperador = self::verficaOperadorExiste($row["DSLOGINECO"]);
            
            $query = $this->connection->prepare(
                "DELETE FROM DADOSFORNECOMMERCE WHERE CDFORNECED = :CDFORNECED AND DSLOGINECO = :DSLOGINECO"
            );
            $query->execute(array(
                "CDFORNECED" => $row["CDFORNECED"],
                "DSLOGINECO" => $row["DSLOGINECO"]
            ));
            
            self::deleteOrganizacaooper($row);
            self::deleteControlsenha($row);
            self::deleteOperador($row);
            
            /**Verifica se operador realmente foi deletado, pois caso seja um operador
             * que ja existia no sistema, o mesmo não pode ser deletado.*/
            $confirmaDeleteOperador = self::verficaOperadorExiste($row["DSLOGINECO"]);
            if(!$confirmaDeleteOperador) {
                self::deletePerfilAcessoParc($cdoperador);
                self::deleteParceiro($cdoperador);
            }
            
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch (\Exception $e){
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvarSenha($row) {
        $query = $this->connection->prepare(
            "SELECT CDOPERFORN FROM DADOSFORNECOMMERCE WHERE DSLOGINECO = :DSLOGINECO"
        );
        $query->execute(array("DSLOGINECO"  => $row["DSLOGINECO"]));
        $dataSet = $query->fetchAll();
        
        if(sizeof($dataSet) > 0) {
            $query = $this->connection->prepare(
                "INSERT INTO CONTROLSENHA (CDOPERADOR, DTALTSENHA, CDSENHALT, IDATUCTRLSENHA, NRORG) VALUES (:CDOPERFORN, SYSDATE, :CDSENHALT, :IDATUCTRLSENHA, :NRORG)"
               );
            $query->execute(array(
                "CDOPERFORN"     => $dataSet[0]["CDOPERFORN"],
                "CDSENHALT"      => $this->utilities->getDelphiPasswordEncrypted($row["CDSENHAWEB"]),
                "IDATUCTRLSENHA" => 'R',
                "NRORG"          => $this->environment->getNrOrgTrab()
            ));
            
            $query = $this->connection->prepare(
                "UPDATE DADOSFORNECOMMERCE SET CDSENHA = :CDSENHA, CDSENHAWEB = :CDSENHAWEB WHERE CDOPERFORN = :CDOPERFORN"
            );
            $query->execute(array(
                "CDOPERFORN" => $dataSet[0]["CDOPERFORN"],
                "CDSENHA"    => $this->utilities->getDelphiPasswordEncrypted($row["CDSENHAWEB"]),
                "CDSENHAWEB" => $this->criptografia->retornaSenhaWeb($row["CDSENHAWEB"])
            ));
        }
    }
    
    public function salvarOperador($row) {
       $query = $this->connection->prepare(
            "INSERT INTO 
                OPERADOR (
                    CDOPERADOR, DSEMAILOPER, NRORGTRAB, NRPARCNEGOCIO,
                    CDSENHOPER, CDSENHAOPERWEB, NMOPERADOR
                )
                SELECT 
                    :CDOPERADOR, :DSLOGINECO, :NRORG, :NRPARCNEGOCIO,
                    :CDSENHA, :CDSENHAWEB, :NMOPERADOR
                FROM 
                    DUAL
                WHERE 
                    NOT EXISTS (
                        SELECT 
                            1
                        FROM 
                            OPERADOR
                        WHERE 
                            DSEMAILOPER = :DSLOGINECO
                    )"
        );
        
        $query->execute(array(
            "CDOPERADOR"    => $row["CDOPERADOR"],
            "DSLOGINECO"    => $row["DSLOGINECO"],
            "NRORG"         => $this->environment->getNrOrg(),
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "CDSENHAWEB"    => $this->criptografia->retornaSenhaWeb($row["CDSENHAWEB"]),
            "CDSENHA"       => $this->utilities->getDelphiPasswordEncrypted($row["CDSENHAWEB"]),
            "NMOPERADOR"    => $row["NMOPERADOR"]
        ));
    }
    
    public function deleteParceiro($cdoperador) {
        if($cdoperador) {
            $query = $this->entityManager->getConnection()->prepare(
                "DELETE FROM PARCNEGOCIO WHERE NRINSCRICAOPARC LIKE '%" . $cdoperador . "'"
            );
            
            $query->execute();
        }
        
    }  
    
    public function deleteOperador($row) {
        $query = $this->connection->prepare(
            "DELETE FROM OPERADOR WHERE CDOPERADOR LIKE '1099%' AND DSEMAILOPER = :DSLOGINECO"
        );
        $query->execute(array(
            "DSLOGINECO" => $row["DSLOGINECO"]
        ));
    }
    
    public function deleteOrganizacaooper($row) {

        $query = $this->entityManager->getConnection()->prepare(
            "DELETE FROM ORGANIZACAOOPER WHERE CDOPERADOR IN( SELECT CDOPERADOR FROM OPERADOR WHERE CDOPERADOR LIKE '1099%' AND DSEMAILOPER = :DSLOGINECO)"
        );
        
        $query->execute(array(
            "DSLOGINECO" => $row["DSLOGINECO"]
        ));
    
    } 
    
    public function enviarEmail(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $row['CDFILIAL'] = self::retornaCdfilial();
        
        $this->serviceEmail->enviarEmail($row, $response);
    }
        
    public function geraNumOperador($row)
    {
        $row['CDOPERFORN'] = '1099' . str_pad(mt_rand(1,99999999),8,'0',STR_PAD_LEFT);
    }
}